rm(list=ls())
## ---------------------------------------------------------------------------------------
## Download and install marima
#download.file("https://02417.compute.dtu.dk/material/marima2_0.1.tar.gz", "marima2_0.1.tar.gz")
#install.packages("marima2_0.1.tar.gz", repos=NULL)
library(marima2)

## ---------------------------------------------------------------------------------------
# Load the functions in the folder
sapply(dir("functions/", full.names=TRUE), source)


## ---------------------------------------------------------------------------------------
# A simulation function for ARMA simulation, use model as arima.sim, i.e. flip sign of phi (into ar) coefficients
sim <- function(model, n, nburnin=100){
  n <- n + nburnin
  # Take the ar and ma part
  ar <- model$ar
  ma <- model$ma
  # The order (i.e. the number of lags)
  p <- length(ar)
  q <- length(ma)
  # The vector for the simulation result
  y <- numeric(n)
  # Generate the random normal values
  eps <- rnorm(n)
  # Run the simulation
  for(i in (max(p,q)+1):n){
    y[i] <- eps[i] + sum(ar * y[i-(1:p)]) + sum(ma * eps[i-(1:q)])
  }
  # Return without the burn-in period
  return(y[(nburnin+1):n])
}


## ---------------------------------------------------------------------------------------
# Simulate an ARMA(1,1) model
# Test it by comparing
model <- list(ar=c(0.3), ma=c(0.5))
set.seed(12)
sim(model, 10, nburnin=100)
set.seed(12)
arima.sim(model, 10, n.start=100)


## ---------------------------------------------------------------------------------------
# A bit longer series to work with
n <- 200
set.seed(12)
y <- sim(model, n)
plot(y, type="l")

# Estimate with arima
(fitArima <- arima(y, order=c(1,0,1)))
# Validation ok?
resArima <- validate(fitArima)

# Estimate with marima
fitMarima <- marima("y ~ AR(1) + MA(1)", data=data.frame(y=y))
summary(fitMarima)
# Validation ok?
resMarima <- validate(fitMarima)

# Difference in residuals
plot(resArima - resMarima)


## ---------------------------------------------------------------------------------------
# CHALLENGE: CAN YOU FIND A WAY TO FIT AN ARMAX!? (like in marima)
## ---------------------------------------------------------------------------------------

## ---------------------------------------------------------------------------------------
# First an AR model with lm
# Add the lags to a data.frame
maxlag <- 5
D <- data.frame(y=y)
for(i in 1:maxlag){
  tmp <- data.frame(lagvec(D$y, i))
  names(tmp) <- paste0("y.l",i)
  D <- cbind(D, tmp)
}
names(D)
head(D)


## ---------------------------------------------------------------------------------------
# TASK1: GO ON FIT AN AR model with lm: what order? do some validation
## ---------------------------------------------------------------------------------------
fit <- lm(y ~ ?, D)
res <- validate(fit)


# When you are done, join Zoom to show your solution
# Paste this into your webbrowser: https://dtudk.zoom.us/j/5075042773


## ---------------------------------------------------------------------------------------
# TASK2: How can we actually add the MA(1) part!?
#        Why not put the LAGGED residuals from the AR fit and to make it an ARMA!?
## ---------------------------------------------------------------------------------------
lm(y ~ ?, D)
