rm(list=ls())
library(marima2)
sapply(dir("functions/", full.names=TRUE), source)


## ---------------------------------------------------------------------------------------
# A simulation function for ARMA simulation, use model as arima.sim, i.e. flip sign of phi (into ar) coefficients
simarmaxmulti <- function(ar11, ar12, coef1, ma1, ar22, ar21, coef2, ma2, x1, x2, sd1=1, sd2=2, nburnin=100){
  # init
  n <- length(x1) + nburnin
  x1 <- c(rep(x1[1],nburnin), x1)
  x2 <- c(rep(x2[1],nburnin), x2)
  # The vectors for the simulation result
  y1 <- numeric(n)
  y2 <- numeric(n)
  # Generate the random normal values
  eps1 <- rnorm(n, sd=sd1)
  eps2 <- rnorm(n, sd=sd2)
  # Run the simulation
  for(i in 2:n){
    y1[i] <- ar11 * y1[i-1]  +  ar12 * y2[i-1]  +  coef1 * x1[i-1]  +  ma1 * eps1[i-1]  +  eps1[i]
    y2[i] <- ar22 * y2[i-1]  +  ar21 * y1[i-1]  +  coef2 * x2[i-1]  +  ma2 * eps2[i-1]  +  eps2[i]
  }
  # Return without the burn-in period
  return( data.frame(x1=x1, x2=x2, y1=y1, y2=y2)[(nburnin+1):n, ] )
}


## ---------------------------------------------------------------------------------------
# Simulate an ARMAX(1,1) model two coupled output, with a step inputs

# Test it by comparing
n <- 100
# Make a step input
x1 <- c(rep(0,50), rep(10,n-50))
x2 <- c(0,0,rep(10,18), rep(0,n-20))
#set.seed(21)
D <- simarmaxmulti(ar11=0.7, ar12=0.2, ma1=0.3, coef1=3,
                   ar22=0.7, ar21=0.2, ma2=0.3, coef2=3, x1, x2, sd1=1, sd2=2)
# Plot to see it
par(mfrow=c(2,1))
plot(D$y1, type="b")
lines(x1, type="s", col=2)
plot(D$y2, type="b")
lines(x2, type="s", col=2)


## ---------------------------------------------------------------------------------------
# Estimate with arima (must lag x1)
# Can't do multi-output coupled models...

# Estimate with marima
fitMarima <- marima("y1 ~ AR(1) + y2(1) + x1(1) + MA(1)",
                    "y2 ~ AR(1) + y1(1) + x2(1) + MA(1)", data=D)
summary(fitMarima)
# Validation ok?
resMarima <- validate(fitMarima)




## ---------------------------------------------------------------------------------------
# CHALLENGE: CAN YOU FIND A WAY TO FIT AN ARMAX!? (like in marima)
## ---------------------------------------------------------------------------------------


